@@ -3,13 +3,14 @@
 # or resolved (into an object) from resolveobj.
 
 package PDL::PP::CType;
+use strict;
+use warnings;
 use Carp;
 
 # new PDL::PP::CType(resolveobj,str)
 
 sub new {
 	my $this = bless {},shift;
-	$this->{Resolve} = shift;
 	$this->parsefrom(shift) if @_;
 	return $this;
 }
@@ -21,9 +22,9 @@ sub stripptrs {
 		return [];
 	}
 	# Now, recall the different C syntaxes. First priority is a pointer:
-	return ["PTR", @{$this->stripptrs($1)}] if $str =~ /^\s*\*(.*)$/;
+	return [["PTR"], @{$this->stripptrs($1)}] if $str =~ /^\s*\*(.*)$/;
 	return $this->stripptrs($1) if $str =~ /^\s*\(.*\)\s*$/; # XXX Should try to see if a funccall.
-	return ["ARR($2)", @{$this->stripptrs($1)}] if $str =~ /^(.*)\[([^]]+)\]\s*$/;
+	return [["ARR",$2], @{$this->stripptrs($1)}] if $str =~ /^(.*)\[([^]]*)\]\s*$/;
 	Carp::confess("Invalid C type '$str'");
 }
 
@@ -39,15 +40,15 @@ sub parsefrom {
 sub get_decl {
 	my($this,$name,$opts) = @_;
 	for(@{$this->{Chain}}) {
-		if($_ eq "PTR") {$name = "*$name"}
-		elsif($_ =~/^ARR\((.*)\)$/) {
+		my ($type, $arg) = @$_;
+		if($type eq "PTR") {$name = "*$name"}
+		elsif($type eq "ARR") {
 			if($opts->{VarArrays2Ptrs}) {
 				$name = "*$name";
 			} else {
-				$name = "($name)[$1]";
+				$name = "($name)[$arg]";
 			}
-		}
-		else { confess("Invalid decl") }
+		} else { confess("Invalid decl @$_") }
 	}
 	return "$this->{Base} $name";
 }
@@ -65,20 +66,23 @@ sub get_copy {
 	my $code = $this->get_malloc($to,$from);
 	return "($to) = ($from);" if !defined $code; # pointer
 	my ($deref0,$deref1,$prev,$close) = ($from,$to);
+        my $no = 0;
 	for(@{$this->{Chain}}) {
-		if($_ eq "PTR") {confess("Cannot copy pointer, must be array");}
-		elsif($_ =~/^ARR\((.*)\)$/) {
+		my ($type, $arg) = @$_;
+		if($type eq "PTR") {confess("Cannot copy pointer, must be array");}
+		elsif($type eq "ARR") {
 			$no++;
-			$prev .= "
+			$arg = "$this->{ProtoName}_count" if $this->is_array;
+			$prev .= PDL::PP::pp_line_numbers(__LINE__-1, "
 			  if(!$deref0) {$deref1=0;}
 			  else {int __malloc_ind_$no;
 				for(__malloc_ind_$no = 0;
-					__malloc_ind_$no < $1;
-					__malloc_ind_$no ++) {";
+					__malloc_ind_$no < $arg;
+					__malloc_ind_$no ++) {");
 			$deref0 = $deref0."[__malloc_ind_$no]";
 			$deref1 = $deref1."[__malloc_ind_$no]";
 			$close .= "}}";
-		} else { confess("Invalid decl $_") }
+		} else { confess("Invalid decl @$_") }
 	}
 	$code .= "$prev $deref1 = $deref0; $close";
 	return $code;
@@ -86,7 +90,7 @@ sub get_copy {
 
 sub get_free {
 	my($this,$from) = @_;
-	return "" if !@{$this->{Chain}} or $this->{Chain}[0] eq 'PTR';
+	return "" if !@{$this->{Chain}} or $this->{Chain}[0][0] eq 'PTR';
 	return "free($from);" if $this->{Base} =~ /^\s*char\s*$/;
 	return "SvREFCNT_dec($from);" if $this->{Base} =~ /^\s*SV\s*$/;
 	croak("Can only free one layer!\n") if @{$this->{Chain}} > 1;
@@ -95,34 +99,30 @@ sub get_free {
 
 sub need_malloc {
 	my($this) = @_;
-	grep /(ARR|PTR)/, @{$this->{Chain}};
+	grep /(ARR|PTR)/, map $_->[0], @{$this->{Chain}};
 }
 
 # returns with the array string - undef if a pointer not needing malloc
 sub get_malloc {
-	my($this,$assignto) = @_;
-	my $str = "{";
-	my $deref = "$assignto";
-	my $prev = undef;
-	my $close = undef;
-	my $no = 0;
-	for(@{$this->{Chain}}) {
-		if($_ eq "PTR") {return}
-		elsif($_ =~/^ARR\((.*)\)$/) {
-			$str .= "$prev $assignto =
-				malloc(sizeof(* $assignto) * $1);
-				";
-			$no++;
-			$prev = "{int __malloc_ind_$no;
-				for(__malloc_ind_$no = 0;
-					__malloc_ind_$no < $1;
-					__malloc_ind_$no ++) {";
-			$deref = $deref."[__malloc_ind_$no]";
-			$close .= "}}";
-		} else { confess("Invalid decl $_") }
-	}
-	$str .= "}";
-	return $str;
+  my($this,$assignto) = @_;
+  my $str = "";
+  for(@{$this->{Chain}}) {
+    my ($type, $arg) = @$_;
+    if($type eq "PTR") {return}
+    elsif($type eq "ARR") {
+      $arg = "$this->{ProtoName}_count" if $this->is_array;
+      $str .= PDL::PP::pp_line_numbers(__LINE__-1, "$assignto = malloc(sizeof(*$assignto) * $arg);\n");
+    } else { confess("Invalid decl (@$_)") }
+  }
+  return $str;
+}
+
+sub is_array {
+  my ($self) = @_;
+  @{$self->{Chain}} &&
+    @{$self->{Chain}[0]} &&
+    $self->{Chain}[0][0] eq 'ARR' &&
+    !$self->{Chain}[0][1];
 }
 
 1;
